# Generated by Django 2.2.15 on 2020-09-12 11:16

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("afat", "0007_auto_20200826_1537"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="afat",
            options={
                "default_permissions": (),
                "verbose_name": "FAT",
                "verbose_name_plural": "FATs",
            },
        ),
        migrations.AlterModelOptions(
            name="afatdellog",
            options={
                "default_permissions": (),
                "verbose_name": "Delete Log",
                "verbose_name_plural": "Delete Logs",
            },
        ),
        migrations.AlterModelOptions(
            name="afatlink",
            options={
                "ordering": ("-afattime",),
                "permissions": (
                    (
                        "manage_afat",
                        "Can manage the Another Fleet Activity Tracking module",
                    ),
                    ("stats_corp_own", "Can see own corp stats"),
                    ("stats_corp_other", "Can see stats of other corps."),
                    (
                        "stats_char_other",
                        "Can see stats of characters not associated with current user.",
                    ),
                ),
                "verbose_name": "FAT Link",
                "verbose_name_plural": "FAT Links",
            },
        ),
        migrations.AlterModelOptions(
            name="afatlinktype",
            options={
                "default_permissions": (),
                "verbose_name": "FAT Link Type",
                "verbose_name_plural": "FAT Link Types",
            },
        ),
    ]



"""
Admin pages configuration
"""

# Django
from django.contrib import admin, messages
from django.db.models import Count

# Alliance Auth AFAT
from afat.models import AFat, AFatLink, AFatLinkType, AFatLog


def custom_filter(title):
    """
    Defining custom filter titles
    :param title:
    :type title:
    :return:
    :rtype:
    """

    class Wrapper(admin.FieldListFilter):
        """
        Wrapper
        """

        def expected_parameters(self):
            """
            Expected parameters
            :return:
            :rtype:
            """

            pass

        def choices(self, changelist):
            """
            Choices
            :param changelist:
            :type changelist:
            :return:
            :rtype:
            """

            pass

        def __new__(cls, *args, **kwargs):
            """
            __new__
            :param args:
            :type args:
            :param kwargs:
            :type kwargs:
            """

            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title

            return instance

    return Wrapper


# Register your models here.
@admin.register(AFatLink)
class AFatLinkAdmin(admin.ModelAdmin):
    """
    Config for fat link model
    """

    list_select_related = ("link_type",)
    list_display = (
        "afattime",
        "creator",
        "fleet",
        "link_type",
        "is_esilink",
        "hash",
        "number_of_fats",
    )
    list_filter = ("is_esilink", ("link_type__name", custom_filter(title="fleet type")))
    ordering = ("-afattime",)
    search_fields = (
        "link_type__name",
        "hash",
        "fleet",
        "creator__profile__main_character__character_name",
        "creator__username",
    )

    def get_queryset(self, request):
        """
        Get the queryset
        :param request:
        :type request:
        :return:
        :rtype:
        """
        queryset = super().get_queryset(request)
        queryset = queryset.annotate(
            _number_of_fats=Count("afats", distinct=True),
        )

        return queryset

    @admin.display(ordering="_number_of_fats")
    def number_of_fats(self, obj):
        """
        Return the number of FATs per FAT Link
        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj._number_of_fats


@admin.register(AFat)
class AFatAdmin(admin.ModelAdmin):
    """
    Config for fat model
    """

    list_display = ("character", "system", "shiptype", "afatlink")
    list_filter = ("character", "system", "shiptype")
    ordering = ("-character",)
    search_fields = (
        "character__character_name",
        "system",
        "shiptype",
        "afatlink__fleet",
        "afatlink__hash",
    )


@admin.register(AFatLinkType)
class AFatLinkTypeAdmin(admin.ModelAdmin):
    """
    Config for fatlinktype model
    """

    list_display = ("id", "_name", "_is_enabled")
    list_filter = ("is_enabled",)
    ordering = ("name",)

    @admin.display(description="Fleet Type", ordering="name")
    def _name(self, obj):
        """
        Rewrite name
        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.name

    @admin.display(description="Is Enabled", boolean=True, ordering="is_enabled")
    def _is_enabled(self, obj):
        """
        Rewrite is_enabled
        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.is_enabled

    actions = (
        "activate",
        "deactivate",
    )

    @admin.action(description="Activate selected fleet type(s)")
    def activate(self, request, queryset):
        """
        Mark fleet type as active
        :param request:
        :type request:
        :param queryset:
        :type queryset:
        :return:
        :rtype:
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                obj.is_enabled = True
                obj.save()

                notifications_count += 1
            except:  # noqa: E722
                failed += 1

        if failed:
            messages.error(request, f"Failed to activate {failed} fleet types")

        if queryset.count() - failed > 0:
            messages.success(request, f"Activated {notifications_count} fleet type(s)")

    @admin.action(description="Deactivate selected fleet type(s)")
    def deactivate(self, request, queryset):
        """
        Mark fleet type as inactive
        :param request:
        :type request:
        :param queryset:
        :type queryset:
        :return:
        :rtype:
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                obj.is_enabled = False
                obj.save()

                notifications_count += 1
            except:  # noqa: E722
                failed += 1

        if failed:
            messages.error(request, f"Failed to deactivate {failed} fleet types")

        if queryset.count() - failed > 0:
            messages.success(
                request, f"Deactivated {notifications_count} fleet type(s)"
            )


@admin.register(AFatLog)
class AFatLogAdmin(admin.ModelAdmin):
    """
    Config for admin log
    """

    list_display = ("log_time", "log_event", "user", "fatlink_hash", "log_text")
    ordering = ("-log_time",)
    readonly_fields = ("log_time", "log_event", "user", "fatlink_hash", "log_text")
    list_filter = ("log_event",)
    search_fields = (
        "fatlink_hash",
        "user__profile__main_character__character_name",
        "user__username",
    )



"""
URL configuration
"""

# Django
from django.urls import path

# Alliance Auth AFAT
from afat.views import dashboard, fatlinks, logs, statistics

app_name: str = "afat"

urlpatterns = [
    # Dashboard
    path("", dashboard.overview, name="dashboard"),
    # Stats main page
    path("statistics/", statistics.overview, name="statistics_overview"),
    path("statistics/(<int:year>/", statistics.overview, name="statistics_overview"),
    # Stats corp
    path(
        "statistics/corporation/",
        statistics.corporation,
        name="statistics_corporation",
    ),
    path(
        "statistics/corporation/<int:corpid>/",
        statistics.corporation,
        name="statistics_corporation",
    ),
    path(
        "statistics/corporation/<int:corpid>/<int:year>/",
        statistics.corporation,
        name="statistics_corporation",
    ),
    path(
        "statistics/corporation/<int:corpid>/<int:year>/<int:month>/",
        statistics.corporation,
        name="statistics_corporation",
    ),
    # Stats char
    path("statistics/character/", statistics.character, name="statistics_character"),
    path(
        "statistics/character/<int:charid>/",
        statistics.character,
        name="statistics_character",
    ),
    path(
        "statistics/character/<int:charid>/<int:year>/<int:month>/",
        statistics.character,
        name="statistics_character",
    ),
    # Stats alliance
    path("statistics/alliance/", statistics.alliance, name="statistics_alliance"),
    path(
        "statistics/alliance/<int:allianceid>/",
        statistics.alliance,
        name="statistics_alliance",
    ),
    path(
        "statistics/alliance/<int:allianceid>/<int:year>/",
        statistics.alliance,
        name="statistics_alliance",
    ),
    path(
        "statistics/alliance/<int:allianceid>/<int:year>/<int:month>/",
        statistics.alliance,
        name="statistics_alliance",
    ),
    # Fat links list actions
    path("fatlinks/", fatlinks.overview, name="fatlinks_overview"),
    path("fatlinks/<int:year>/", fatlinks.overview, name="fatlinks_overview"),
    # Fat link actions
    path("fatlink/add/", fatlinks.add_fatlink, name="fatlinks_add_fatlink"),
    path(
        "fatlinks/link/create/esi-fatlink/",
        fatlinks.create_esi_fatlink,
        name="fatlinks_create_esi_fatlink",
    ),
    path(
        "fatlink/create/esi-fatlink/callback/<str:fatlink_hash>/",
        fatlinks.create_esi_fatlink_callback,
        name="fatlinks_create_esi_fatlink_callback",
    ),
    path(
        "fatlink/create/clickable-fatlink/",
        fatlinks.create_clickable_fatlink,
        name="fatlinks_create_clickable_fatlink",
    ),
    path(
        "fatlink/<str:fatlink_hash>/details/",
        fatlinks.details_fatlink,
        name="fatlinks_details_fatlink",
    ),
    path(
        "fatlink/<str:fatlink_hash>/delete/",
        fatlinks.delete_fatlink,
        name="fatlinks_delete_fatlink",
    ),
    path(
        "fatlink/<str:fatlink_hash>/stop-esi-tracking/",
        fatlinks.close_esi_fatlink,
        name="fatlinks_close_esi_fatlink",
    ),
    path(
        "fatlink/<str:fatlink_hash>/re-open/",
        fatlinks.reopen_fatlink,
        name="fatlinks_reopen_fatlink",
    ),
    # Fat actions
    path(
        "fatlink/<str:fatlink_hash>/register/",
        fatlinks.add_fat,
        name="fatlinks_add_fat",
    ),
    path(
        "fatlink/<str:fatlink_hash>/fat/<int:fat_id>/delete/",
        fatlinks.delete_fat,
        name="fatlinks_delete_fat",
    ),
    # Log actions
    path("logs/", logs.overview, name="logs_overview"),
    # Ajax calls :: Dashboard
    path(
        "ajax/dashboard/fatlinks/recent/",
        dashboard.ajax_get_recent_fatlinks,
        name="dashboard_ajax_get_recent_fatlinks",
    ),
    path(
        "ajax/dashboard/fats/recent/character/<int:charid>/",
        dashboard.ajax_recent_get_fats_by_character,
        name="dashboard_ajax_get_recent_fats_by_character",
    ),
    # Ajax calls :: Fat links
    path(
        "ajax/fatlinks/fatlinks/year/<int:year>/",
        fatlinks.ajax_get_fatlinks_by_year,
        name="fatlinks_ajax_get_fatlinks_by_year",
    ),
    path(
        "ajax/fatlinks/fatlink/<str:fatlink_hash>/fats/",
        fatlinks.ajax_get_fats_by_fatlink,
        name="fatlinks_ajax_get_fats_by_fatlink",
    ),
    # Ajax calls :: Logs
    path("ajax/logs/", logs.ajax_get_logs, name="logs_ajax_get_logs"),
]



# flake8: noqa
"""
Scripts generates large amount of fat links for load testing

This script can be executed directly from shell.
"""

# Standard Library
import os
import sys
from pathlib import Path

myauth_dir = Path(__file__).parent.parent.parent.parent.parent / "myauth"
sys.path.insert(0, str(myauth_dir))

# Django
import django

# init and setup django project
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "myauth.settings.local")
django.setup()

# Standard Library
import datetime as dt
import random

# Django
from django.contrib.auth.models import User
from django.utils.timezone import now

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth (External Libs)
from app_utils.helpers import random_string

# Alliance Auth AFAT
from afat.models import AFat, AFatLink, AFatLinkType, AFatLog
from afat.tests.fixtures.utils import RequestStub
from afat.utils import write_log

LINKS_NUMBER = 1000


characters = list(EveCharacter.objects.all())

print(
    f"Adding {LINKS_NUMBER:,} FAT links "
    f"with up to {len(characters)} characters each"
)

user = User.objects.first()
creator = user.profile.main_character
link_type, _ = AFatLinkType.objects.get_or_create(name="Generated Fleet")

for _ in range(LINKS_NUMBER):
    fat_link = AFatLink.objects.create(
        fleet=f"Generated Fleet #{random.randint(1, 1000000000)}",
        hash=random_string(20),
        creator=user,
        character=creator,
        link_type=link_type,
        afattime=now() - dt.timedelta(days=random.randint(0, 180)),
    )
    write_log(
        request=RequestStub(user),
        log_event=AFatLog.Event.CREATE_FATLINK,
        log_text=(
            f'ESI FAT link with name "{fat_link.fleet}"'
            f"{link_type} was created by {user}"
        ),
        fatlink_hash=fat_link.hash,
    )

    for character in random.sample(characters, k=random.randint(1, len(characters))):
        AFat.objects.create(
            character_id=character.id,
            afatlink=fat_link,
            system="Jita",
            shiptype="Ibis",
        )

    print(".", end="", flush=True)


print("")
print("DONE")



"""
Migrate the old manual FAT log into the new log table
"""

# Django
from django.core.management.base import BaseCommand

# Alliance Auth AFAT
from afat.models import AFatLog, ManualAFat


def get_input(text) -> str:
    """
    Wrapped input to enable import
    :param text:
    :type text:
    :return:
    :rtype:
    """

    return input(text)


class Command(BaseCommand):
    """
    Migrate manual FAT log
    """

    help = "Migrating the old Manual FAT log into the new log table"

    def _migrate_manual_fat_log(self) -> None:
        """
        Start the migration
        :return:
        :rtype:
        """

        manual_fat_logs = ManualAFat.objects.all()

        if manual_fat_logs.count() > 0:
            for manual_log in manual_fat_logs:
                if manual_log.created_at is not None:
                    afat_log = AFatLog()

                    afat_log.user_id = manual_log.creator_id
                    afat_log.log_time = manual_log.created_at
                    afat_log.log_event = AFatLog.Event.MANUAL_FAT
                    afat_log.log_text = (
                        f"Pilot {manual_log.character} manually added. "
                        f"(Migrated from old Manual FAT log)"
                    )
                    afat_log.fatlink_hash = manual_log.afatlink.hash
                    afat_log.save()

                manual_log.delete()

        self.stdout.write(self.style.SUCCESS("Migration complete!"))

    def handle(self, *args, **options):
        """
        Ask before running ...
        :param args:
        :type args:
        :param options:
        :type options:
        :return:
        :rtype:
        """

        self.stdout.write(
            "This will migrate the old Manual FAT log into the new log table. "
            "Migrated entries will be removed from the old Manual FAT log to "
            "prevent duplicates."
        )

        user_input = get_input("Are you sure you want to proceed? (yes/no)?")

        if user_input == "yes":
            self.stdout.write("Starting import. Please stand by.")
            self._migrate_manual_fat_log()
        else:
            self.stdout.write(self.style.WARNING("Aborted."))



# Generated by Django 2.0.8 on 2018-09-11 08:31

# Django
from django.conf import settings
from django.db import migrations, models

# Alliance Auth AFAT
import afat.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("afat", "0002_manualfat"),
    ]

    operations = [
        migrations.CreateModel(
            name="AFatDelLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("deltype", models.BooleanField(default=0)),
                ("string", models.CharField(max_length=100)),
                (
                    "remover",
                    models.ForeignKey(
                        on_delete=models.SET(afat.models.get_sentinel_user),
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.AlterModelOptions(
            name="afatlink",
            options={
                "permissions": (
                    (
                        "manage_afat",
                        "Can manage the Another Fleet Activity Tracking module",
                    ),
                    ("stats_corp_own", "Can see own corp stats"),
                    ("stats_corp_other", "Can see stats of other corps."),
                    (
                        "stats_char_other",
                        "Can see stats of characters not associated with current user.",
                    ),
                )
            },
        ),
        migrations.RemoveField(
            model_name="afat",
            name="station",
        ),
        migrations.RemoveField(
            model_name="afat",
            name="user",
        ),
        migrations.AlterField(
            model_name="afat",
            name="shiptype",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="afat",
            name="system",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="afatlink",
            name="fleet",
            field=models.CharField(max_length=254, null=True),
        ),
    ]



"""
Logs related views
"""

# Django
from django.contrib.auth.decorators import login_required
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse, JsonResponse
from django.shortcuts import render

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# Alliance Auth AFAT
from afat import __title__
from afat.app_settings import AFAT_DEFAULT_LOG_DURATION
from afat.helper.views_helper import convert_logs_to_dict
from afat.models import AFatLink, AFatLog

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required()
@permissions_required(("afat.manage_afat", "afat.log_view"))
def overview(request: WSGIRequest) -> HttpResponse:
    """
    Logs view
    :param request:
    :type request:
    :return:
    :rtype:
    """

    logger.info(f"Log view called by {request.user}")

    context = {"log_duration": AFAT_DEFAULT_LOG_DURATION}

    return render(request, "afat/view/logs/logs_overview.html", context=context)


@login_required()
@permissions_required(("afat.manage_afat", "afat.log_view"))
def ajax_get_logs(request: WSGIRequest) -> JsonResponse:
    """
    Ajax call :: get all log entries
    :param request:
    :type request:
    :return:
    :rtype:
    """

    logs = AFatLog.objects.select_related("user", "user__profile__main_character").all()
    fatlink_hashes = set(AFatLink.objects.values_list("hash", flat=True))

    log_rows = [
        convert_logs_to_dict(log=log, fatlink_exists=log.fatlink_hash in fatlink_hashes)
        for log in logs
    ]

    return JsonResponse(log_rows, safe=False)



# Generated by Django 3.1.1 on 2020-10-03 22:22

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("afat", "0010_permissoins_update_20201002_1909"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="manualafat",
            options={
                "verbose_name": "Manual FAT Log",
                "verbose_name_plural": "Manual FAT Logs",
            },
        ),
    ]



# Generated by Django 2.2.14 on 2020-08-26 15:37

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("afat", "0006_auto_20200820_2013"),
    ]

    operations = [
        migrations.AlterField(
            model_name="afatlinktype",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
    ]



"""
Statistics related views
"""

# Standard Library
import calendar
from collections import OrderedDict
from datetime import datetime

# Django
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import Permission
from django.core.handlers.wsgi import WSGIRequest
from django.db.models import Count
from django.http import HttpResponse
from django.shortcuts import redirect, render
from django.utils.safestring import mark_safe
from django.utils.translation import gettext

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required
from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# Alliance Auth AFAT
from afat import __title__
from afat.helper.views_helper import (
    characters_with_permission,
    get_random_rgba_color,
    user_has_any_perms,
)
from afat.models import AFat
from afat.utils import get_or_create_alliance_info, get_or_create_corporation_info

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required()
@permission_required("afat.basic_access")
def overview(request: WSGIRequest, year: int = None) -> HttpResponse:
    """
    Statistics main view
    :param request:
    :type request:
    :param year:
    :type year:
    :return:
    :rtype:
    """

    if year is None:
        year = datetime.now().year

    user_can_see_other_corps = False

    if user_has_any_perms(
        request.user,
        ["afat.stats_corporation_other", "afat.manage_afat"],
    ):
        user_can_see_other_corps = True
        basic_access_permission = Permission.objects.select_related("content_type").get(
            content_type__app_label="afat", codename="basic_access"
        )

        characters_with_access = characters_with_permission(basic_access_permission)

        data = {"No Alliance": [1]}
        sanity_check = {}

        # First create the alliance keys in our dict
        for character_with_access in characters_with_access:
            if character_with_access.alliance_name is not None:
                data[character_with_access.alliance_name] = [
                    character_with_access.alliance_id
                ]

        # Now append the alliance keys
        for character_with_access in characters_with_access:
            corp_id = character_with_access.corporation_id
            corp_name = character_with_access.corporation_name

            # if corp_id not in sanity_check.keys():
            if corp_id not in sanity_check:
                if character_with_access.alliance_name is None:
                    data["No Alliance"].append((corp_id, corp_name))
                else:
                    data[character_with_access.alliance_name].append(
                        (corp_id, corp_name)
                    )

            sanity_check[corp_id] = corp_id

    elif request.user.has_perm("afat.stats_corporation_own"):
        data = [
            (
                request.user.profile.main_character.corporation_id,
                request.user.profile.main_character.corporation_name,
            )
        ]
    else:
        data = None

    months = _calculate_year_stats(request, year)

    context = {
        "data": data,
        "charstats": months,
        "year": year,
        "year_current": datetime.now().year,
        "year_prev": int(year) - 1,
        "year_next": int(year) + 1,
        "user_can_see_other_corps": user_can_see_other_corps,
    }

    logger.info(f"Statistics overview called by {request.user}")

    return render(request, "afat/view/statistics/statistics_overview.html", context)


def _calculate_year_stats(request, year) -> list:
    """
    Calculate and return year statistics.
    """

    months = []
    characters = EveCharacter.objects.filter(character_ownership__user=request.user)

    for char in characters:
        character_fats_in_year = (
            AFat.objects.filter(afatlink__afattime__year=year)
            .filter(character=char)
            .values("afatlink__afattime__month")
            .annotate(fat_count=Count("id"))
        )

        # Only if there are FATs for this year for the character
        if character_fats_in_year:
            character_fats_per_month = {
                int(result["afatlink__afattime__month"]): result["fat_count"]
                for result in character_fats_in_year
            }

            # Sort by month
            character_fats_per_month = dict(
                sorted(character_fats_per_month.items(), key=lambda item: item[0])
            )

            months.append(
                (char.character_name, character_fats_per_month, char.character_id)
            )

    # Return sorted by character name
    return sorted(months, key=lambda x: x[0])


@login_required()
@permission_required("afat.basic_access")
def character(
    request: WSGIRequest, charid: int, year: int = None, month: int = None
) -> HttpResponse:
    """
    Character statistics view
    :param request:
    :type request:
    :param charid:
    :type charid:
    :param year:
    :type year:
    :param month:
    :type month:
    :return:
    :rtype:
    """

    eve_character = EveCharacter.objects.get(character_id=charid)
    valid = [
        char.character for char in CharacterOwnership.objects.filter(user=request.user)
    ]

    can_view_character = True

    # Check if the user can view other corporation's statistics or manage AFAT
    if eve_character not in valid and not user_has_any_perms(
        request.user,
        [
            "afat.stats_corporation_other",
            "afat.manage_afat",
        ],
    ):
        can_view_character = False

    # Check if the user if by any chance in the same corporation as the character
    # and can view own corporation statistics
    if (
        eve_character not in valid
        and eve_character.corporation_id
        == request.user.profile.main_character.corporation_id
        and request.user.has_perm("afat.stats_corporation_own")
    ):
        can_view_character = True

    # If the user cannot view the character's statistics, send him home
    if can_view_character is False:
        messages.warning(
            request,
            mark_safe(
                gettext(
                    "<h4>Warning!</h4>"
                    "<p>You do not have permission to view "
                    "statistics for this character.</p>"
                )
            ),
        )

        return redirect("afat:dashboard")

    if not month or not year:
        messages.error(
            request,
            mark_safe(
                gettext("<h4>Warning!</h4><p>Date information not complete!</p>")
            ),
        )

        return redirect("afat:dashboard")

    fats = AFat.objects.filter(
        character__character_id=charid,
        afatlink__afattime__month=month,
        afatlink__afattime__year=year,
    )

    # Data for Ship Type Pie Chart
    data_ship_type = {}

    for fat in fats:
        if fat.shiptype in data_ship_type:
            continue

        data_ship_type[fat.shiptype] = fats.filter(shiptype=fat.shiptype).count()

    colors = []

    for _ in data_ship_type:
        bg_color_str = get_random_rgba_color()
        colors.append(bg_color_str)

    data_ship_type = [
        # Ship type can be None, so we need to convert to string here
        list(str(key) for key in data_ship_type),
        list(data_ship_type.values()),
        colors,
    ]

    # Data for by Time Line Chart
    data_time = {}

    for i in range(0, 24):
        data_time[i] = fats.filter(afatlink__afattime__hour=i).count()

    data_time = [
        list(data_time.keys()),
        list(data_time.values()),
        [get_random_rgba_color()],
    ]

    context = {
        "character": eve_character,
        "month": month,
        "month_current": datetime.now().month,
        "month_prev": int(month) - 1,
        "month_next": int(month) + 1,
        "year": year,
        "year_current": datetime.now().year,
        "year_prev": int(year) - 1,
        "year_next": int(year) + 1,
        "data_ship_type": data_ship_type,
        "data_time": data_time,
        "fats": fats,
    }

    month_name = calendar.month_name[int(month)]
    logger.info(
        f"Character statistics for {eve_character} ({month_name} {year}) "
        f"called by {request.user}"
    )

    return render(request, "afat/view/statistics/statistics_character.html", context)


@login_required()
@permissions_required(
    ("afat.stats_corporation_other", "afat.stats_corporation_own", "afat.manage_afat")
)
def corporation(
    request: WSGIRequest, corpid: int = 0000, year: int = None, month: int = None
) -> HttpResponse:
    """
    Corp statistics view
    :param request:
    :type request:
    :param corpid:
    :type corpid:
    :param year:
    :type year:
    :param month:
    :type month:
    :return:
    :rtype:
    """

    if not year:
        year = datetime.now().year

    # Check character has permission to view other corp stats
    if int(request.user.profile.main_character.corporation_id) != int(corpid):
        if not user_has_any_perms(
            request.user,
            ["afat.stats_corporation_other", "afat.manage_afat"],
        ):
            messages.warning(
                request,
                mark_safe(
                    gettext(
                        "<h4>Warning!</h4>"
                        "<p>You do not have permission to view statistics "
                        "for that corporation.</p>"
                    )
                ),
            )

            return redirect("afat:dashboard")

    corp = get_or_create_corporation_info(corporation_id=corpid)
    corp_name = corp.corporation_name

    if not month:
        months = []

        for i in range(1, 13):
            corp_fats = AFat.objects.filter(
                character__corporation_id=corpid,
                afatlink__afattime__month=i,
                afatlink__afattime__year=year,
            ).count()

            avg_fats = 0
            if corp.member_count > 0:
                avg_fats = corp_fats / corp.member_count

            if corp_fats > 0:
                months.append((i, corp_fats, round(avg_fats, 2)))

        context = {
            "corporation": corp.corporation_name,
            "months": months,
            "corpid": corpid,
            "year": year,
            "year_current": datetime.now().year,
            "year_prev": int(year) - 1,
            "year_next": int(year) + 1,
            "type": 0,
        }

        return render(
            request,
            "afat/view/statistics/statistics_corporation_year_overview.html",
            context,
        )

    fats = AFat.objects.filter(
        afatlink__afattime__month=month,
        afatlink__afattime__year=year,
        character__corporation_id=corpid,
    )

    characters = EveCharacter.objects.filter(corporation_id=corpid)

    # Data for Stacked Bar Graph
    # (label, color, [list of data for stack])
    data = {}

    for fat in fats:
        # if fat.shiptype in data.keys():
        if fat.shiptype in data:
            continue

        data[fat.shiptype] = {}

    chars = []

    for fat in fats:
        if fat.character.character_name in chars:
            continue

        chars.append(fat.character.character_name)

    for key, ship_type in data.items():
        for char in chars:
            ship_type[char] = 0

    for fat in fats:
        data[fat.shiptype][fat.character.character_name] += 1

    data_stacked = []

    for key, value in data.items():
        stack = []
        stack.append(key)
        stack.append(get_random_rgba_color())
        stack.append([])

        data_ = stack[2]

        for char in chars:
            data_.append(value[char])

        stack.append(data_)
        data_stacked.append(tuple(stack))

    data_stacked = [chars, data_stacked]

    # Data for By Time
    data_time = {}

    for i in range(0, 24):
        data_time[i] = fats.filter(afatlink__afattime__hour=i).count()

    data_time = [
        list(data_time.keys()),
        list(data_time.values()),
        [get_random_rgba_color()],
    ]

    # Data for By Weekday
    data_weekday = []

    for i in range(1, 7):
        data_weekday.append(fats.filter(afatlink__afattime__iso_week_day=i).count())

    data_weekday = [
        ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"],
        data_weekday,
        [get_random_rgba_color()],
    ]

    chars = {}

    for char in characters:
        fat_c = fats.filter(character_id=char.id).count()
        chars[char.character_name] = (fat_c, char.character_id)

    context = {
        "corp": corp,
        "corporation": corp.corporation_name,
        "month": month,
        "month_current": datetime.now().month,
        "month_prev": int(month) - 1,
        "month_next": int(month) + 1,
        "year": year,
        "year_current": datetime.now().year,
        "year_prev": int(year) - 1,
        "year_next": int(year) + 1,
        "data_stacked": data_stacked,
        "data_time": data_time,
        "data_weekday": data_weekday,
        "chars": chars,
    }

    month_name = calendar.month_name[int(month)]
    logger.info(
        f"Corporation statistics for {corp_name} ({month_name} {year}) "
        f"called by {request.user}"
    )

    return render(request, "afat/view/statistics/statistics_corporation.html", context)


@login_required()
@permissions_required(("afat.stats_corporation_other", "afat.manage_afat"))
def alliance(
    request: WSGIRequest, allianceid: int, year: int = None, month: int = None
) -> HttpResponse:
    """
    Alliance statistics view
    :param request:
    :type request:
    :param allianceid:
    :type allianceid:
    :param year:
    :type year:
    :param month:
    :type month:
    :return:
    :rtype:
    """

    if not year:
        year = datetime.now().year

    if allianceid == "000":
        allianceid = None

    if allianceid is not None:
        ally = get_or_create_alliance_info(alliance_id=allianceid)
        alliance_name = ally.alliance_name
    else:
        ally = None
        alliance_name = "No Alliance"

    if not month:
        months = []

        for i in range(1, 13):
            ally_fats = AFat.objects.filter(
                character__alliance_id=allianceid,
                afatlink__afattime__month=i,
                afatlink__afattime__year=year,
            ).count()

            if ally_fats > 0:
                months.append((i, ally_fats))

        context = {
            "alliance": alliance_name,
            "months": months,
            "allianceid": allianceid,
            "year": year,
            "year_current": datetime.now().year,
            "year_prev": int(year) - 1,
            "year_next": int(year) + 1,
            "type": 1,
        }

        return render(
            request,
            "afat/view/statistics/statistics_alliance_year_overview.html",
            context,
        )

    if not month or not year:
        messages.error(
            request,
            mark_safe(gettext("<h4>Error!</h4><p>Date information incomplete.</p>")),
        )

        return redirect("afat:dashboard")

    fats = AFat.objects.filter(
        character__alliance_id=allianceid,
        afatlink__afattime__month=month,
        afatlink__afattime__year=year,
    )

    corporations = EveCorporationInfo.objects.filter(alliance=ally)

    # Data for Ship Type Pie Chart
    data_ship_type = {}

    for fat in fats:
        if fat.shiptype in data_ship_type:
            continue

        data_ship_type[fat.shiptype] = fats.filter(shiptype=fat.shiptype).count()

    colors = []

    for _ in data_ship_type:
        bg_color_str = get_random_rgba_color()
        colors.append(bg_color_str)

    data_ship_type = [
        # Ship type can be None, so we need to convert to string here
        list(str(key) for key in data_ship_type),
        list(data_ship_type.values()),
        colors,
    ]

    # Fats by corp and ship type?
    data = {}

    for fat in fats:
        if fat.shiptype in data:
            continue

        data[fat.shiptype] = {}

    corps = []

    for fat in fats:
        if fat.character.corporation_name in corps:
            continue

        corps.append(fat.character.corporation_name)

    for key, ship_type in data.items():
        for corp in corps:
            ship_type[corp] = 0

    for fat in fats:
        data[fat.shiptype][fat.character.corporation_name] += 1

    if None in data:
        data["Unknown"] = data[None]
        data.pop(None)

    data_stacked = []

    for key, value in data.items():
        stack = []
        stack.append(key)
        stack.append(get_random_rgba_color())
        stack.append([])

        data_ = stack[2]

        for corp in corps:
            data_.append(value[corp])

        stack.append(data_)
        data_stacked.append(tuple(stack))

    data_stacked = [corps, data_stacked]

    # Avg fats by corp
    data_avgs = {}

    for corp in corporations:
        c_fats = fats.filter(character__corporation_id=corp.corporation_id).count()
        avg = c_fats / corp.member_count
        data_avgs[corp.corporation_name] = round(avg, 2)

    data_avgs = OrderedDict(sorted(data_avgs.items(), key=lambda x: x[1], reverse=True))
    data_avgs = [
        list(data_avgs.keys()),
        list(data_avgs.values()),
        get_random_rgba_color(),
    ]

    # Fats by Time
    data_time = {}

    for i in range(0, 24):
        data_time[i] = fats.filter(afatlink__afattime__hour=i).count()

    data_time = [
        list(data_time.keys()),
        list(data_time.values()),
        [get_random_rgba_color()],
    ]

    # Fats by weekday
    data_weekday = []

    for i in range(1, 7):
        data_weekday.append(fats.filter(afatlink__afattime__iso_week_day=i).count())

    data_weekday = [
        ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"],
        data_weekday,
        [get_random_rgba_color()],
    ]

    # Corp list
    corps = {}

    for corp in corporations:
        c_fats = fats.filter(character__corporation_id=corp.corporation_id).count()
        avg = c_fats / corp.member_count
        corps[corp] = (corp.corporation_id, c_fats, round(avg, 2))

    corps = OrderedDict(sorted(corps.items(), key=lambda x: x[1][2], reverse=True))

    context = {
        "alliance": alliance_name,
        "ally": ally,
        "month": month,
        "month_current": datetime.now().month,
        "month_prev": int(month) - 1,
        "month_next": int(month) + 1,
        "year": year,
        "year_current": datetime.now().year,
        "year_prev": int(year) - 1,
        "year_next": int(year) + 1,
        "data_stacked": data_stacked,
        "data_avgs": data_avgs,
        "data_time": data_time,
        "data_weekday": data_weekday,
        "corps": corps,
        "data_ship_type": data_ship_type,
    }

    month_name = calendar.month_name[int(month)]
    logger.info(
        f"Alliance statistics for {alliance_name} ({month_name} {year}) "
        f"called by {request.user}"
    )

    return render(request, "afat/view/statistics/statistics_alliance.html", context)



"""
Fat links related views
"""

# Standard Library
from datetime import datetime, timedelta

# Django
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.db import IntegrityError
from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils import timezone
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required
from allianceauth.eveonline.models import EveCharacter
from allianceauth.eveonline.providers import provider
from allianceauth.services.hooks import get_extension_logger
from esi.decorators import token_required
from esi.models import Token

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# Alliance Auth AFAT
from afat import __title__
from afat.app_settings import (
    AFAT_DEFAULT_FATLINK_EXPIRY_TIME,
    AFAT_DEFAULT_FATLINK_REOPEN_DURATION,
    AFAT_DEFAULT_FATLINK_REOPEN_GRACE_TIME,
)
from afat.forms import (
    AFatClickFatForm,
    AFatEsiFatForm,
    AFatManualFatForm,
    FatLinkEditForm,
)
from afat.helper.fatlinks import get_esi_fleet_information_by_user
from afat.helper.time import get_time_delta
from afat.helper.views_helper import convert_fatlinks_to_dict, convert_fats_to_dict
from afat.models import (
    AFat,
    AFatLink,
    AFatLinkType,
    AFatLog,
    ClickAFatDuration,
    get_hash_on_save,
)
from afat.providers import esi
from afat.tasks import process_fats
from afat.utils import get_or_create_character, write_log

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required()
@permission_required("afat.basic_access")
def overview(request: WSGIRequest, year: int = None) -> HttpResponse:
    """
    Fat links view
    :param request:
    :type request:
    :param year:
    :type year:
    :return:
    :rtype:
    """

    if year is None:
        year = datetime.now().year

    context = {
        "year": year,
        "year_current": datetime.now().year,
        "year_prev": int(year) - 1,
        "year_next": int(year) + 1,
    }

    logger.info(f"FAT link list called by {request.user}")

    return render(request, "afat/view/fatlinks/fatlinks_overview.html", context)


@login_required()
@permission_required("afat.basic_access")
def ajax_get_fatlinks_by_year(request: WSGIRequest, year: int) -> JsonResponse:
    """
    Ajax call :: get all FAT links for a given year
    :param request:
    :type request:
    :param year:
    :type year:
    :return:
    :rtype:
    """

    fatlinks = (
        AFatLink.objects.select_related_default()
        .filter(afattime__year=year)
        .annotate_afats_count()
    )

    fatlink_rows = [
        convert_fatlinks_to_dict(
            request=request,
            fatlink=fatlink,
            close_esi_redirect=reverse("afat:fatlinks_overview"),
        )
        for fatlink in fatlinks
    ]

    return JsonResponse(fatlink_rows, safe=False)


@login_required()
@permissions_required(("afat.manage_afat", "afat.add_fatlink"))
def add_fatlink(request: WSGIRequest) -> HttpResponse:
    """
    Add fat link view
    :param request:
    :type request:
    :return:
    :rtype:
    """

    link_types_configured = False
    link_types_count = AFatLinkType.objects.all().count()

    if link_types_count > 0:
        link_types_configured = True

    context = {
        "link_types_configured": link_types_configured,
        "default_expiry_time": AFAT_DEFAULT_FATLINK_EXPIRY_TIME,
        "esi_fleet": get_esi_fleet_information_by_user(request.user),
        "esi_fatlink_form": AFatEsiFatForm(),
        "manual_fatlink_form": AFatClickFatForm(),
    }

    logger.info(f"Add FAT link view called by {request.user}")

    return render(request, "afat/view/fatlinks/fatlinks_add_fatlink.html", context)


@login_required()
@permissions_required(("afat.manage_afat", "afat.add_fatlink"))
def create_clickable_fatlink(
    request: WSGIRequest,
) -> HttpResponseRedirect:
    """
    Create clickable fat link
    :param request:
    :type request:
    :return:
    :rtype:
    """

    if request.method == "POST":
        form = AFatClickFatForm(request.POST)

        if form.is_valid():
            fatlink_hash = get_hash_on_save()

            fatlink = AFatLink()
            fatlink.fleet = form.cleaned_data["name"]

            if form.cleaned_data["type"] is not None:
                fatlink.link_type = form.cleaned_data["type"]

            fatlink.creator = request.user
            fatlink.hash = fatlink_hash
            fatlink.afattime = timezone.now()
            fatlink.save()

            dur = ClickAFatDuration()
            dur.fleet = AFatLink.objects.get(hash=fatlink_hash)
            dur.duration = form.cleaned_data["duration"]
            dur.save()

            # Writing DB log
            fleet_type = (
                f" (Fleet Type: {fatlink.link_type.name})" if fatlink.link_type else ""
            )

            write_log(
                request=request,
                log_event=AFatLog.Event.CREATE_FATLINK,
                log_text=(
                    f'FAT link with name "{form.cleaned_data["name"]}"{fleet_type} and '
                    f'a duration of {form.cleaned_data["duration"]} minutes was created'
                ),
                fatlink_hash=fatlink.hash,
            )

            logger.info(
                f'FAT link "{fatlink_hash}" with name '
                f'"{form.cleaned_data["name"]}"{fleet_type} and a duration '
                f'of {form.cleaned_data["duration"]} minutes was created '
                f"by {request.user}"
            )

            messages.success(
                request,
                mark_safe(
                    _(
                        "<h4>Success!</h4>"
                        "<p>Clickable FAT Link Created!</p>"
                        "<p>Make sure to give your fleet members the link to "
                        "click so that they get credit for this fleet.</p>"
                    )
                ),
            )

            return redirect("afat:fatlinks_details_fatlink", fatlink_hash=fatlink_hash)

        messages.error(
            request,
            mark_safe(
                _(
                    "<h4>Error!</h4>"
                    "<p>Something went wrong when attempting "
                    "to submit your clickable FAT Link.</p>"
                )
            ),
        )

        return redirect("afat:fatlinks_add_fatlink")

    messages.warning(
        request,
        mark_safe(
            _(
                "<h4>Warning!</h4>"
                '<p>You must fill out the form on the "Add FAT Link" '
                "page to create a clickable FAT Link</p>"
            )
        ),
    )

    return redirect("afat:fatlinks_add_fatlink")


@login_required()
@permissions_required(("afat.manage_afat", "afat.add_fatlink"))
@token_required(scopes=["esi-fleets.read_fleet.v1"])
def create_esi_fatlink_callback(
    request: WSGIRequest, token, fatlink_hash: str
) -> HttpResponseRedirect:
    """
    Helper :: create ESI link (callback, used when coming back from character selection)
    :param request:
    :type request:
    :param token:
    :type token:
    :param fatlink_hash:
    :type fatlink_hash:
    :return:
    :rtype:
    """

    # Check if there is a fleet
    try:
        required_scopes = ["esi-fleets.read_fleet.v1"]
        esi_token = Token.get_token(token.character_id, required_scopes)

        fleet_from_esi = esi.client.Fleets.get_characters_character_id_fleet(
            character_id=token.character_id, token=esi_token.valid_access_token()
        ).result()
    except Exception:
        # Not in a fleet
        messages.warning(
            request,
            mark_safe(
                _(
                    "<h4>Warning!</h4>"
                    "<p>To use the ESI function, you need to be in fleet and you need "
                    "to be the fleet boss! You can create a clickable FAT link and "
                    "share it, if you like.</p>"
                )
            ),
        )

        # Return to "Add FAT Link" view
        return redirect("afat:fatlinks_add_fatlink")

    # check if this character already has a fleet
    creator_character = EveCharacter.objects.get(character_id=token.character_id)
    registered_fleets_for_creator = AFatLink.objects.select_related_default().filter(
        is_esilink=True,
        is_registered_on_esi=True,
        character__character_name=creator_character.character_name,
    )

    fleet_already_registered = False
    character_has_registered_fleets = False
    registered_fleets_to_close = []

    if registered_fleets_for_creator.count() > 0:
        character_has_registered_fleets = True

        for registered_fleet in registered_fleets_for_creator:
            if registered_fleet.esi_fleet_id == fleet_from_esi["fleet_id"]:
                # Character already has a fleet
                fleet_already_registered = True
            else:
                registered_fleets_to_close.append(
                    {"registered_fleet": registered_fleet}
                )

    # If the FC already has a fleet, and it is the same as already registered,
    # just throw a warning
    if fleet_already_registered is True:
        messages.warning(
            request,
            mark_safe(
                _(
                    f'<h4>Warning!</h4><p>Fleet with ID "{fleet_from_esi["fleet_id"]}" for your character {creator_character.character_name} has already been registered and pilots joining this fleet are automatically tracked.</p>'
                )
            ),
        )

        # Return to "Add FAT Link" view
        return redirect("afat:fatlinks_add_fatlink")

    # If it's a new fleet, remove all former registered fleets, if there are any
    if (
        character_has_registered_fleets is True
        and fleet_already_registered is False
        and len(registered_fleets_to_close) > 0
    ):
        for registered_fleet_to_close in registered_fleets_to_close:
            reason = (
                f"FC has opened a new fleet with the "
                f"character {creator_character.character_name}"
            )

            logger.info(
                f"Closing ESI FAT link with hash "
                f'"{registered_fleet_to_close["registered_fleet"].hash}". '
                f"Reason: {reason}"
            )

            registered_fleet_to_close["registered_fleet"].is_registered_on_esi = False
            registered_fleet_to_close["registered_fleet"].save()

    # Check if we deal with the fleet boss here
    try:
        esi_fleet_member = esi.client.Fleets.get_fleets_fleet_id_members(
            fleet_id=fleet_from_esi["fleet_id"],
            token=esi_token.valid_access_token(),
        ).result()
    except Exception:
        messages.warning(
            request,
            mark_safe(
                _(
                    "<h4>Warning!</h4>"
                    "<p>Not Fleet Boss! Only the fleet boss can utilize the ESI "
                    "function. You can create a clickable FAT link and share it, "
                    "if you like.</p>"
                )
            ),
        )

        # Return to "Add FAT Link" view
        return redirect("afat:fatlinks_add_fatlink")

    creator_character = EveCharacter.objects.get(character_id=token.character_id)

    # Create the fat link
    fatlink = AFatLink(
        afattime=timezone.now(),
        fleet=request.session["fatlink_form__name"],
        creator=request.user,
        character=creator_character,
        hash=fatlink_hash,
        is_esilink=True,
        is_registered_on_esi=True,
        esi_fleet_id=fleet_from_esi["fleet_id"],
    )

    # Add fleet type, if there is any
    if request.session["fatlink_form__type"] is not None:
        fatlink.link_type_id = request.session["fatlink_form__type"]

    # Save it
    fatlink.save()

    # Writing DB log
    fleet_type = ""
    if fatlink.link_type:
        fleet_type = f"(Fleet Type: {fatlink.link_type.name})"

    write_log(
        request=request,
        log_event=AFatLog.Event.CREATE_FATLINK,
        log_text=(
            f'ESI FAT link with name "{request.session["fatlink_form__name"]}" '
            f"{fleet_type} was created by {request.user}"
        ),
        fatlink_hash=fatlink.hash,
    )

    logger.info(
        f'ESI FAT link "{fatlink_hash}" with name '
        f'"{request.session["fatlink_form__name"]}"{fleet_type} '
        f"was created by {request.user}"
    )

    # Clear session
    del request.session["fatlink_form__name"]
    del request.session["fatlink_form__type"]

    # Process fleet members in background
    process_fats.delay(
        data_list=esi_fleet_member, data_source="esi", fatlink_hash=fatlink_hash
    )

    messages.success(
        request,
        mark_safe(
            _(
                "<h4>Success!</h4>"
                "<p>FAT Link Created!</p>"
                "<p>FATs have been queued, they may take a few mins to show up.</p>"
                "<p>Pilots who join later will be automatically added until you "
                "close or leave the fleet in-game.</p>"
            )
        ),
    )

    return redirect("afat:fatlinks_details_fatlink", fatlink_hash=fatlink_hash)


@login_required()
def create_esi_fatlink(
    request: WSGIRequest,
) -> HttpResponseRedirect:
    """
    Create ESI fat link
    :param request:
    :type request:
    :return:
    :rtype:
    """

    fatlink_form = AFatEsiFatForm(request.POST)

    if fatlink_form.is_valid():
        fatlink_hash = get_hash_on_save()

        fatlink_type = None
        if fatlink_form.cleaned_data["type_esi"]:
            fatlink_type_from_form = fatlink_form.cleaned_data["type_esi"]
            fatlink_type = fatlink_type_from_form.pk

        request.session["fatlink_form__name"] = fatlink_form.cleaned_data["name_esi"]
        request.session["fatlink_form__type"] = fatlink_type

        return redirect(
            "afat:fatlinks_create_esi_fatlink_callback", fatlink_hash=fatlink_hash
        )

    messages.error(
        request,
        mark_safe(
            _(
                "<h4>Error!</h4>"
                "<p>Something went wrong when attempting to "
                "submit your ESI FAT Link.</p>"
            )
        ),
    )

    return redirect("afat:fatlinks_add_fatlink")


@login_required()
@permission_required("afat.basic_access")
@token_required(
    scopes=[
        "esi-location.read_location.v1",
        "esi-location.read_ship_type.v1",
        "esi-location.read_online.v1",
    ]
)
def add_fat(
    request: WSGIRequest, token, fatlink_hash: str = None
) -> HttpResponseRedirect:
    """
    Click fat link helper
    :param request:
    :type request:
    :param token:
    :type token:
    :param fatlink_hash:
    :type fatlink_hash:
    :return:
    :rtype:
    """

    if fatlink_hash is None:
        messages.warning(
            request,
            mark_safe(_("<h4>Warning!</h4><p>No FAT link hash provided.</p>")),
        )

        return redirect("afat:dashboard")

    try:
        fleet = AFatLink.objects.get(hash=fatlink_hash, is_esilink=False)
    except AFatLink.DoesNotExist:
        messages.warning(
            request,
            mark_safe(_("<h4>Warning!</h4><p>The hash provided is not valid.</p>")),
        )

        return redirect("afat:dashboard")

    dur = ClickAFatDuration.objects.get(fleet=fleet)
    now = timezone.now() - timedelta(minutes=dur.duration)

    if now >= fleet.afattime:
        messages.warning(
            request,
            mark_safe(
                _(
                    "<h4>Warning!</h4>"
                    "<p>Sorry, that FAT Link is expired. "
                    "If you were on that fleet, contact your FC about "
                    "having your FAT manually added.</p>"
                )
            ),
        )

        return redirect("afat:dashboard")

    character = EveCharacter.objects.get(character_id=token.character_id)

    try:
        required_scopes = [
            "esi-location.read_location.v1",
            "esi-location.read_online.v1",
            "esi-location.read_ship_type.v1",
        ]
        esi_token = Token.get_token(token.character_id, required_scopes)
    except Exception:
        messages.warning(
            request,
            mark_safe(
                _(
                    f"<h4>Warning!</h4><p>There was an issue with the ESI token for {character.character_name}. Please try again.</p>"
                )
            ),
        )

        return redirect("afat:dashboard")

    # Check if character is online
    character_online = esi.client.Location.get_characters_character_id_online(
        character_id=token.character_id, token=esi_token.valid_access_token()
    ).result()

    if character_online["online"] is True:
        # Character location
        location = esi.client.Location.get_characters_character_id_location(
            character_id=token.character_id,
            token=esi_token.valid_access_token(),
        ).result()

        # Current ship
        ship = esi.client.Location.get_characters_character_id_ship(
            character_id=token.character_id,
            token=esi_token.valid_access_token(),
        ).result()

        # System information
        system = esi.client.Universe.get_universe_systems_system_id(
            system_id=location["solar_system_id"]
        ).result()["name"]

        ship_name = provider.get_itemtype(ship["ship_type_id"]).name

        try:
            AFat(
                afatlink=fleet, character=character, system=system, shiptype=ship_name
            ).save()
        except IntegrityError:
            messages.warning(
                request,
                mark_safe(
                    _(
                        f"<h4>Warning!</h4><p>A FAT already exists for the selected character ({character.character_name}) and fleet combination.</p>"
                    )
                ),
            )
        else:
            if fleet.fleet is not None:
                fleet_name = fleet.fleet
            else:
                fleet_name = fleet.hash

            messages.success(
                request,
                mark_safe(
                    _(
                        f"<h4>Success!</h4><p>FAT registered for {character.character_name} at {fleet_name}</p>"
                    )
                ),
            )

            logger.info(
                f'Participation for fleet "{fleet_name}" registered for '
                f"pilot {character.character_name}"
            )
    else:
        messages.warning(
            request,
            mark_safe(
                _(
                    f"<h4>Warning!</h4><p>Cannot register the fleet participation for {character.character_name}. The character needs to be online.</p>"
                )
            ),
        )

    return redirect("afat:dashboard")


@login_required()
@permissions_required(("afat.manage_afat", "afat.add_fatlink"))
def details_fatlink(request: WSGIRequest, fatlink_hash: str) -> HttpResponse:
    """
    Fat link view
    :param request:
    :type request:
    :param fatlink_hash:
    :type fatlink_hash:
    :return:
    :rtype:
    """

    try:
        link = AFatLink.objects.select_related_default().get(hash=fatlink_hash)
    except AFatLink.DoesNotExist:
        messages.warning(
            request,
            mark_safe(_("<h4>Warning!</h4><p>The hash provided is not valid.</p>")),
        )

        return redirect("afat:dashboard")

    if request.method == "POST":
        fatlink_edit_form = FatLinkEditForm(request.POST)
        manual_fat_form = AFatManualFatForm(request.POST)

        if fatlink_edit_form.is_valid():
            link.fleet = fatlink_edit_form.cleaned_data["fleet"]
            link.save()

            # Writing DB log
            write_log(
                request=request,
                log_event=AFatLog.Event.CHANGE_FATLINK,
                log_text=f'FAT link changed. Fleet name was set to "{link.fleet}"',
                fatlink_hash=link.hash,
            )

            logger.info(
                f'FAT link with hash "{link.hash}" changed. '
                f'Fleet name was set to "{link.fleet}" by {request.user}'
            )

            messages.success(
                request,
                mark_safe(
                    _("<h4>Success!</h4><p>Fleet name successfully changed.</p>")
                ),
            )
        elif manual_fat_form.is_valid():
            character_name = manual_fat_form.cleaned_data["character"]
            system = manual_fat_form.cleaned_data["system"]
            shiptype = manual_fat_form.cleaned_data["shiptype"]
            character = get_or_create_character(name=character_name)

            if character is not None:
                afat_object, created = AFat.objects.get_or_create(
                    afatlink=link,
                    character=character,
                    defaults={
                        "system": system,
                        "shiptype": shiptype,
                    },
                )

                if created is True:
                    messages.success(
                        request,
                        mark_safe(
                            _(
                                f"<h4>Success!</h4><p>Manual FAT processed.<br>{character.character_name} has been added flying a {shiptype} in {system}</p>"
                            )
                        ),
                    )

                    # Writing DB log
                    write_log(
                        request=request,
                        log_event=AFatLog.Event.MANUAL_FAT,
                        log_text=(
                            f"Pilot {character.character_name} "
                            f"flying a {shiptype} was manually added"
                        ),
                        fatlink_hash=link.hash,
                    )

                    logger.info(
                        f"Pilot {character.character_name} flying a {shiptype} was "
                        "manually added to FAT link with "
                        f'hash "{link.hash}" by {request.user}'
                    )
                else:
                    messages.info(
                        request,
                        mark_safe(
                            _(
                                f"<h4>Information</h4><p>Pilot is already registered for this FAT link.</p><p>Name: {character.character_name}<br>System: {afat_object.system}<br>Ship: {afat_object.shiptype}</p>"
                            )
                        ),
                    )
            else:
                messages.error(
                    request,
                    mark_safe(
                        _(
                            "<h4>Oh No!</h4>"
                            "<p>Manual FAT processing failed! "
                            "The character name you entered was not found.</p>"
                        )
                    ),
                )
        else:
            messages.error(
                request,
                mark_safe(_("<h4>Oh No!</h4><p>Something went wrong!</p>")),
            )

    logger.info(f'FAT link "{fatlink_hash}" details view called by {request.user}')

    # Let's see if the link is still valid or has expired already and can be re-opened
    # and FATs can be manually added
    # (only possible for 24 hours after creating the FAT link)
    link_ongoing = True
    link_can_be_reopened = False
    link_expires = None
    manual_fat_can_be_added = False

    # Time dependant settings
    try:
        dur = ClickAFatDuration.objects.get(fleet=link)
    except ClickAFatDuration.DoesNotExist:
        # ESI link
        link_ongoing = False
    else:
        link_expires = link.afattime + timedelta(minutes=dur.duration)
        now = timezone.now()

        if link_expires <= now:
            # Link expired
            link_ongoing = False

            if (
                link.reopened is False
                and get_time_delta(link_expires, now, "minutes")
                < AFAT_DEFAULT_FATLINK_REOPEN_GRACE_TIME
            ):
                link_can_be_reopened = True

        # Manual fat still possible?
        # Only possible if the FAT link has not been re-opened
        # and has been created within the last 24 hours
        if link.reopened is False and get_time_delta(link.afattime, now, "hours") < 24:
            manual_fat_can_be_added = True

    is_clickable_link = False
    if link.is_esilink is False:
        is_clickable_link = True

    if link.is_esilink and link.is_registered_on_esi:
        link_ongoing = True

    context = {
        "link": link,
        "is_esi_link": link.is_esilink,
        "is_clickable_link": is_clickable_link,
        "link_expires": link_expires,
        "link_ongoing": link_ongoing,
        "link_can_be_reopened": link_can_be_reopened,
        "manual_fat_can_be_added": manual_fat_can_be_added,
        "reopen_grace_time": AFAT_DEFAULT_FATLINK_REOPEN_GRACE_TIME,
        "reopen_duration": AFAT_DEFAULT_FATLINK_REOPEN_DURATION,
    }

    return render(request, "afat/view/fatlinks/fatlinks_details_fatlink.html", context)


@login_required()
@permissions_required(("afat.manage_afat", "afat.add_fatlink"))
def ajax_get_fats_by_fatlink(request: WSGIRequest, fatlink_hash) -> JsonResponse:
    """
    Ajax call :: get all FATs for a given FAT link hash
    :param request:
    :type request:
    :param fatlink_hash:
    :type fatlink_hash:
    :return:
    :rtype:
    """

    fats = AFat.objects.select_related_default().filter(afatlink__hash=fatlink_hash)

    fat_rows = [convert_fats_to_dict(request=request, fat=fat) for fat in fats]

    return JsonResponse(fat_rows, safe=False)


@login_required()
@permission_required("afat.manage_afat")
def delete_fatlink(
    request: WSGIRequest, fatlink_hash: str = None
) -> HttpResponseRedirect:
    """
    Delete fat link helper
    :param request:
    :type request:
    :param fatlink_hash:
    :type fatlink_hash:
    :return:
    :rtype:
    """

    if fatlink_hash is None:
        messages.warning(
            request,
            mark_safe(_("<h4>Warning!</h4><p>No FAT Link hash provided.</p>")),
        )

        return redirect("afat:dashboard")

    try:
        link = AFatLink.objects.get(hash=fatlink_hash)
    except AFatLink.DoesNotExist:
        messages.error(
            request,
            mark_safe(
                _(
                    "<h4>Error!</h4>"
                    "<p>The fatlink hash provided is either invalid "
                    "or the fatlink has already been deleted.</p>"
                )
            ),
        )

        return redirect("afat:dashboard")

    AFat.objects.filter(afatlink_id=link.pk).delete()

    link.delete()

    write_log(
        request=request,
        log_event=AFatLog.Event.DELETE_FATLINK,
        log_text="FAT link deleted.",
        fatlink_hash=link.hash,
    )

    messages.success(
        request,
        mark_safe(
            _(
                f'<h4>Success!</h4><p>The FAT Link "{fatlink_hash}" and all associated FATs have been successfully deleted.</p>'
            )
        ),
    )

    logger.info(
        f'Fat link "{fatlink_hash}" and all associated '
        f"FATs have been deleted by {request.user}"
    )

    return redirect("afat:fatlinks_overview")


@login_required()
@permissions_required(("afat.manage_afat", "afat.delete_afat"))
def delete_fat(
    request: WSGIRequest, fatlink_hash: str, fat_id: int
) -> HttpResponseRedirect:
    """
    Delete fat helper
    :param request:
    :type request:
    :param fatlink_hash:
    :type fatlink_hash:
    :param fat_id:
    :type fat_id:
    :return:
    :rtype:
    """

    try:
        link = AFatLink.objects.get(hash=fatlink_hash)
    except AFatLink.DoesNotExist:
        messages.error(
            request,
            mark_safe(
                _(
                    "<h4>Error!</h4>"
                    "<p>The hash provided is either invalid or has been deleted.</p>"
                )
            ),
        )

        return redirect("afat:dashboard")

    try:
        fat_details = AFat.objects.get(pk=fat_id, afatlink_id=link.pk)
    except AFat.DoesNotExist:
        messages.error(
            request,
            mark_safe(_("<h4>Error!</h4><p>The hash and FAT ID do not match.</p>")),
        )

        return redirect("afat:dashboard")

    fat_details.delete()

    write_log(
        request=request,
        log_event=AFatLog.Event.DELETE_FAT,
        log_text=f"The FAT for {fat_details.character.character_name} has been deleted",
        fatlink_hash=link.hash,
    )

    messages.success(
        request,
        mark_safe(
            _(
                f'<h4>Success!</h4><p>The FAT for {fat_details.character.character_name} has been successfully deleted from FAT link "{fatlink_hash}".</p>'
            )
        ),
    )

    logger.info(
        f"The FAT for {fat_details.character.character_name} has "
        f'been deleted from FAT link "{fatlink_hash}" by {request.user}.'
    )

    return redirect("afat:fatlinks_details_fatlink", fatlink_hash=fatlink_hash)


@login_required()
@permissions_required(("afat.manage_afat", "afat.add_fatlink"))
def close_esi_fatlink(request: WSGIRequest, fatlink_hash: str) -> HttpResponseRedirect:
    """
    Ajax call to close an ESI fat link
    :param request:
    :type request:
    :param fatlink_hash:
    :type fatlink_hash:
    :return:
    :rtype:
    """

    try:
        fatlink = AFatLink.objects.get(hash=fatlink_hash)
    except AFatLink.DoesNotExist:
        logger.info(f'ESI FAT link with hash "{fatlink_hash}" does not exist')
    else:
        logger.info(
            f'Closing ESI FAT link with hash "{fatlink_hash}". '
            f"Reason: Closed by manual request"
        )

        fatlink.is_registered_on_esi = False
        fatlink.save()

    default_redirect = reverse("afat:dashboard")
    next_view = request.GET.get("next", default_redirect)

    return HttpResponseRedirect(next_view)


@login_required()
@permissions_required(("afat.manage_afat", "afat.add_fatlink"))
def reopen_fatlink(request: WSGIRequest, fatlink_hash: str) -> HttpResponseRedirect:
    """
    Re-open fat link
    :param request:
    :type request:
    :param fatlink_hash:
    :type fatlink_hash:
    :return:
    :rtype:
    """

    try:
        fatlink_duration = ClickAFatDuration.objects.get(fleet__hash=fatlink_hash)
    except ClickAFatDuration.DoesNotExist:
        messages.error(
            request,
            mark_safe(
                _(
                    "<h4>Error!</h4>"
                    "<p>The hash you provided does not match with any FAT link.</p>"
                )
            ),
        )

        return redirect("afat:dashboard")

    if fatlink_duration.fleet.reopened is False:
        created_at = fatlink_duration.fleet.afattime
        now = datetime.now()

        time_difference_in_minutes = get_time_delta(created_at, now, "minutes")
        new_duration = time_difference_in_minutes + AFAT_DEFAULT_FATLINK_REOPEN_DURATION

        fatlink_duration.duration = new_duration
        fatlink_duration.save()

        fatlink_duration.fleet.reopened = True
        fatlink_duration.fleet.save()

        # writing DB log
        write_log(
            request=request,
            log_event=AFatLog.Event.REOPEN_FATLINK,
            log_text=(
                f"FAT link re-opened for a duration of {AFAT_DEFAULT_FATLINK_REOPEN_DURATION} minutes"
            ),
            fatlink_hash=fatlink_duration.fleet.hash,
        )

        logger.info(
            f'FAT link with hash "{fatlink_hash}" '
            f"re-opened by {request.user} for a "
            f"duration of {AFAT_DEFAULT_FATLINK_REOPEN_DURATION} minutes"
        )

        messages.success(
            request,
            mark_safe(
                _(
                    "<h4>Success!</h4>"
                    "<p>The FAT link has been successfully re-opened.</p>"
                )
            ),
        )
    else:
        messages.warning(
            request,
            mark_safe(
                _(
                    "<h4>Warning!</h4>"
                    "<p>This FAT link has already been re-opened. "
                    "FAT links can be re-opened only once!</p>"
                )
            ),
        )

    return redirect("afat:fatlinks_details_fatlink", fatlink_hash=fatlink_hash)



"""
Import FAT data from alliance auth fat module
"""

# Django
from django.apps import apps
from django.core.management.base import BaseCommand

# Alliance Auth
from allianceauth.fleetactivitytracking.models import Fat, Fatlink

# Alliance Auth AFAT
from afat.models import AFat, AFatLink, AFatLog


def get_input(text) -> str:
    """
    Wrapped input to enable import
    :param text:
    :type text:
    :return:
    :rtype:
    """

    return input(text)


def aa_fat_installed() -> bool:
    """
    Check if native fat is installed
    :return:
    :rtype:
    """

    return apps.is_installed("allianceauth.fleetactivitytracking")


class Command(BaseCommand):
    """
    Initial import of FAT data from AA FAT module
    """

    help = "Imports FAT data from AA FAT module"

    def _import_from_aa_fat(self) -> None:
        """
        Start the import
        :return:
        :rtype:
        """

        # Check if AA FAT is active
        if aa_fat_installed():
            self.stdout.write(
                self.style.SUCCESS("Alliance Auth FAT module is active, let's go!")
            )

            # First, we check if the target tables are empty ...
            current_afat_links_count = AFatLink.objects.all().count()
            current_afat_count = AFat.objects.all().count()

            if current_afat_count > 0 or current_afat_links_count > 0:
                self.stdout.write(
                    self.style.WARNING(
                        "You already have FAT data with the AFAT module. "
                        "Import cannot be continued."
                    )
                )

                return

            aa_fatlinks = Fatlink.objects.all()
            for aa_fatlink in aa_fatlinks:
                self.stdout.write(
                    f"Importing FAT link for fleet '{aa_fatlink.fleet}' with hash "
                    f"'{aa_fatlink.hash}'."
                )

                afatlink = AFatLink()

                afatlink.id = aa_fatlink.id
                afatlink.afattime = aa_fatlink.fatdatetime
                afatlink.fleet = (
                    aa_fatlink.fleet
                    if aa_fatlink.fleet is not None
                    else aa_fatlink.hash
                )
                afatlink.hash = aa_fatlink.hash
                afatlink.creator_id = aa_fatlink.creator_id

                afatlink.save()

                # Write to log table
                log_text = (
                    f"FAT link {aa_fatlink.hash} with name {aa_fatlink.fleet} "
                    f"was created by {aa_fatlink.creator}"
                )

                afatlog = AFatLog()
                afatlog.log_time = aa_fatlink.fatdatetime
                afatlog.log_event = AFatLog.Event.CREATE_FATLINK
                afatlog.log_text = log_text
                afatlog.user_id = aa_fatlink.creator_id
                afatlog.save()

            aa_fats = Fat.objects.all()
            for aa_fat in aa_fats:
                self.stdout.write(f"Importing FATs for FAT link ID '{aa_fat.id}'.")

                afat = AFat()

                afat.id = aa_fat.id
                afat.system = aa_fat.system
                afat.shiptype = aa_fat.shiptype
                afat.character_id = aa_fat.character_id
                afat.afatlink_id = aa_fat.fatlink_id

                afat.save()

            self.stdout.write(
                self.style.SUCCESS(
                    "Import complete! "
                    "You can now deactivate the Alliance Auth FAT "
                    "module in your local.py"
                )
            )
        else:
            self.stdout.write(
                self.style.WARNING(
                    "Alliance Auth FAT module is not active. "
                    "Please make sure you have it in your "
                    "INSTALLED_APPS in your local.py!"
                )
            )

    def handle(self, *args, **options):
        """
        Ask before running ...
        :param args:
        :type args:
        :param options:
        :type options:
        :return:
        :rtype:
        """

        self.stdout.write(
            "Importing all FAT/FAT link data from Alliance Auth's built in FAT module. "
            "This can only be done once during the very first installation. "
            "As soon as you have data collected with your AFAT module, "
            "this import will fail!"
        )

        user_input = get_input("Are you sure you want to proceed? (yes/no)?")

        if user_input == "yes":
            self.stdout.write("Starting import. Please stand by.")
            self._import_from_aa_fat()
        else:
            self.stdout.write(self.style.WARNING("Aborted."))



"""
Test our template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase

# Alliance Auth AFAT
from afat import __version__


class TestAfatFilters(TestCase):
    """
    Test template filters
    """

    def test_month_name_filter(self):
        """
        Test month_name
        :return:
        """

        context = Context({"month": 5})
        template_to_render = Template("{% load filters %} {{ month|month_name }}")

        rendered_template = template_to_render.render(context)

        self.assertInHTML("May", rendered_template)


class TestAfatVersionedStatic(TestCase):
    """
    Test versioned static template tag
    """

    def test_versioned_static(self):
        """
        Test afat_versioned_static
        :return:
        """

        context = Context({"version": __version__})
        template_to_render = Template(
            "{% load afat_versioned_static %}"
            "{% afat_static 'afat/css/allianceauth-afat.min.css' %}"
        )

        rendered_template = template_to_render.render(context)

        self.assertInHTML(
            f'/static/afat/css/allianceauth-afat.min.css?v={context["version"]}',
            rendered_template,
        )



"""
Auth hooks
"""

# Alliance Auth
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

# Alliance Auth AFAT
from afat import urls
from afat.app_settings import AFAT_APP_NAME, AFAT_BASE_URL


class AaAfatMenuItem(MenuItemHook):  # pylint: disable=too-few-public-methods
    """
    This class ensures only authorized users will see the menu entry
    """

    def __init__(self):
        # Setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            AFAT_APP_NAME,
            "fas fa-space-shuttle fa-fw",
            "afat:dashboard",
            navactive=["afat:"],
        )

    def render(self, request):
        """
        Only if the user has access to this app
        :param request:
        :type request:
        :return:
        :rtype:
        """

        if request.user.has_perm("afat.basic_access"):
            return MenuItemHook.render(self, request)

        return ""


@hooks.register("menu_item_hook")
def register_menu():
    """
    Register our menu
    :return:
    :rtype:
    """

    return AaAfatMenuItem()


@hooks.register("url_hook")
def register_url():
    """
    Register our menu link
    :return:
    :rtype:
    """

    return UrlHook(urls, "afat", rf"^{AFAT_BASE_URL}/")



# Standard Library
from unittest.mock import Mock

# Django
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase
from django.urls import reverse

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth (External Libs)
from app_utils.testing import add_character_to_user, create_user_from_evecharacter

# Alliance Auth AFAT
from afat.models import AFat, AFatLink
from afat.tests.fixtures.load_allianceauth import load_allianceauth
from afat.views.dashboard import overview

MODULE_PATH = "afat.views.dashboard"


def response_content_to_str(response) -> str:
    return response.content.decode(response.charset)


class TestDashboard(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        load_allianceauth()

        # given
        cls.character_1001 = EveCharacter.objects.get(character_id=1001)
        cls.character_1002 = EveCharacter.objects.get(character_id=1002)
        cls.character_1101 = EveCharacter.objects.get(character_id=1101)

        cls.user, _ = create_user_from_evecharacter(
            cls.character_1001.character_id, permissions=["afat.basic_access"]
        )

        add_character_to_user(cls.user, cls.character_1101)

        create_user_from_evecharacter(cls.character_1002.character_id)

        cls.afat_link = AFatLink.objects.create(
            fleet="Demo Fleet",
            hash="123",
            creator=cls.user,
            character=cls.character_1001,
        )

    def _page_overview_request(self, user):
        request = self.factory.get(reverse("afat:dashboard"))
        request.user = user

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)

        return overview(request)

    # def test_should_open_page_normally(self):
    #     # when
    #     response = self._page_request(self.user)
    #
    #     # then
    #     self.assertEqual(response.status_code, 200)

    def test_should_only_show_my_chars_and_only_those_with_fat_links(self):
        # given
        AFat.objects.create(character=self.character_1101, afatlink=self.afat_link)
        AFat.objects.create(character=self.character_1002, afatlink=self.afat_link)

        # when
        response = self._page_overview_request(self.user)

        # then
        content = response_content_to_str(response)

        self.assertEqual(response.status_code, 200)
        self.assertIn(
            f'<span id="afat-eve-character-id-{self.character_1101.character_id}">{self.character_1101.character_name}</span>',
            content,
        )
        self.assertNotIn(
            f'<span id="afat-eve-character-id-{self.character_1001.character_id}">{self.character_1001.character_name}</span>',
            content,
        )
        self.assertNotIn(
            f'<span id="afat-eve-character-id-{self.character_1002.character_id}">{self.character_1002.character_name}</span>',
            content,
        )



"""
The forms we use
"""

# Django
from django import forms
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

# Alliance Auth AFAT
from afat.app_settings import AFAT_DEFAULT_FATLINK_EXPIRY_TIME
from afat.models import AFatLinkType


def get_mandatory_form_label_text(text):
    """
    Label text for mandatory form fields
    :param text:
    :type text:
    :return:
    :rtype:
    """

    required_text = _("This field is mandatory")
    required_marker = (
        f'<span aria-label="{required_text}" class="form-required-marker">*</span>'
    )

    return mark_safe(
        f'<span class="form-field-required">{text} {required_marker}</span>'
    )


class AFatEsiFatForm(forms.Form):
    """
    Fat link form
    used to create ESI fatlinks
    """

    name_esi = forms.CharField(
        required=True,
        label=get_mandatory_form_label_text(_("Fleet Name")),
        max_length=255,
        widget=forms.TextInput(attrs={"placeholder": _("Enter fleet name")}),
    )
    type_esi = forms.ModelChoiceField(
        required=False,
        label=_("Fleet Type (optional)"),
        queryset=AFatLinkType.objects.filter(is_enabled=True),
        # empty_label=_("Please select a fleet type"),
    )


class AFatManualFatForm(forms.Form):
    """
    Manual fat form
    """

    character = forms.CharField(
        required=True,
        label=get_mandatory_form_label_text(_("Character Name")),
        max_length=255,
    )
    system = forms.CharField(
        required=True,
        label=get_mandatory_form_label_text(_("System")),
        max_length=100,
    )
    shiptype = forms.CharField(
        required=True,
        label=get_mandatory_form_label_text(_("Ship Type")),
        max_length=100,
    )


class AFatClickFatForm(forms.Form):
    """
    Fat link form
    used to create clickable fatlinks
    """

    name = forms.CharField(
        required=True,
        label=get_mandatory_form_label_text(_("Fleet Name")),
        max_length=255,
        widget=forms.TextInput(attrs={"placeholder": _("Enter fleet name")}),
    )
    type = forms.ModelChoiceField(
        required=False,
        label=_("Fleet Type (optional)"),
        queryset=AFatLinkType.objects.filter(is_enabled=True),
        # empty_label=_("Please select a fleet type"),
    )
    duration = forms.IntegerField(
        required=True,
        label=get_mandatory_form_label_text(_("FAT link expiry time in minutes")),
        min_value=1,
        initial=AFAT_DEFAULT_FATLINK_EXPIRY_TIME,
        widget=forms.TextInput(attrs={"placeholder": _("Expiry time in minutes")}),
    )


class FatLinkEditForm(forms.Form):
    """
    Fat link edit form
    Used in edit view to change the fat link name
    """

    fleet = forms.CharField(
        required=True,
        label=get_mandatory_form_label_text(_("Fleet Name")),
        max_length=255,
    )



"""
The models
"""

# Django
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.db import models, transaction
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.translation import gettext as _

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth AFAT
from afat.managers import AFatLinkManager, AFatManager


def get_sentinel_user() -> User:
    """
    Get user or create one
    :return:
    """

    return User.objects.get_or_create(username="deleted")[0]


def get_hash_on_save() -> str:
    """
    Get the slug
    :return:
    """

    fatlink_hash = get_random_string(length=30)

    while AFatLink.objects.filter(hash=fatlink_hash).exists():
        fatlink_hash = get_random_string(length=30)

    return fatlink_hash


class AaAfat(models.Model):
    """
    Meta model for app permissions
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        AaAfat :: Meta
        """

        managed = False
        default_permissions = ()
        permissions = (
            # can access and register his own participation to a FAT link
            ("basic_access", "Can access the AFAT module"),
            # Can manage the FAT module
            # Has:
            #   » add_fatlink
            #   » change_fatlink
            #   » delete_fatlink
            #   » add_fat
            #   » delete_fat
            ("manage_afat", "Can manage the AFAT module"),
            # Can add a new FAT link
            ("add_fatlink", "Can create FAT Links"),
            # Can see own corp stats
            ("stats_corporation_own", "Can see own corporation statistics"),
            # Can see the stats of all corps
            ("stats_corporation_other", "Can see statistics of other corporations"),
            # Can view the modules log
            ("log_view", "Can view the modules log"),
        )
        verbose_name = "Alliance Auth AFAT"


# AFatLinkType Model (StratOp, ADM, HD etc)
class AFatLinkType(models.Model):
    """
    AFatLinkType
    """

    id = models.AutoField(primary_key=True)

    name = models.CharField(
        max_length=254, help_text="Descriptive name of your fleet type"
    )

    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this fleet type is active or not",
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        AFatLinkType :: Meta
        """

        default_permissions = ()
        verbose_name = "FAT Link Fleet Type"
        verbose_name_plural = "FAT Link Fleet Types"

    def __str__(self) -> str:
        """
        Return the objects string name
        :return:
        :rtype:
        """

        return str(self.name)


# AFatLink Model
class AFatLink(models.Model):
    """
    AFatLink
    """

    afattime = models.DateTimeField(
        default=timezone.now, db_index=True, help_text="When was this FAT link created"
    )

    fleet = models.CharField(
        max_length=254,
        blank=False,
        default=None,
        help_text="The FAT link fleet name",
    )

    hash = models.CharField(
        max_length=254, db_index=True, unique=True, help_text="The FAT link hash"
    )

    creator = models.ForeignKey(
        User,
        related_name="+",
        on_delete=models.SET(get_sentinel_user),
        help_text="Who created the FAT link?",
    )

    character = models.ForeignKey(
        EveCharacter,
        related_name="+",
        on_delete=models.CASCADE,
        default=None,
        null=True,
        help_text="Character this FAT link has been created with",
    )

    link_type = models.ForeignKey(
        AFatLinkType,
        related_name="+",
        on_delete=models.CASCADE,
        null=True,
        help_text="The FAT link fleet type, if it's set",
    )

    is_esilink = models.BooleanField(
        default=False,
        help_text="Whether this FAT link was created via ESI or not",
    )

    is_registered_on_esi = models.BooleanField(
        default=False,
        help_text="Whether this is an ESI fat link is registered on ESI or not",
    )

    esi_fleet_id = models.BigIntegerField(blank=True, null=True)

    reopened = models.BooleanField(
        default=False,
        help_text="Has this FAT link being re-opened?",
    )

    objects = AFatLinkManager()

    class Meta:  # pylint: disable=too-few-public-methods
        """
        AFatLink :: Meta
        """

        default_permissions = ()
        ordering = ("-afattime",)
        verbose_name = "FAT Link"
        verbose_name_plural = "FAT Links"

    def __str__(self) -> str:
        """
        Return the objects string name
        :return:
        :rtype:
        """

        return f"{self.fleet} - {self.hash}"

    @transaction.atomic()
    def save(self, *args, **kwargs):
        """
        Add the hash on save
        :param args:
        :type args:
        :param kwargs:
        :type kwargs:
        """

        try:
            self.hash
        except ObjectDoesNotExist:
            self.hash = get_hash_on_save()
        super().save(*args, **kwargs)

    @property
    def number_of_fats(self):
        """
        Returns the number of registered FATs
        :return:
        :rtype:
        """

        return AFat.objects.filter(afatlink=self).count()


# ClickAFatDuration Model
class ClickAFatDuration(models.Model):
    """
    ClickAFatDuration
    """

    duration = models.PositiveIntegerField()
    fleet = models.ForeignKey(AFatLink, on_delete=models.CASCADE)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        ClickAFatDuration :: Meta
        """

        default_permissions = ()
        verbose_name = "FAT Duration"
        verbose_name_plural = "FAT Durations"


# AFat Model
class AFat(models.Model):
    """
    AFat
    """

    character = models.ForeignKey(
        EveCharacter,
        related_name="afats",
        on_delete=models.CASCADE,
        help_text="Character who registered this FAT",
    )

    afatlink = models.ForeignKey(
        AFatLink,
        related_name="afats",
        on_delete=models.CASCADE,
        help_text="The FAT link the character registered at",
    )

    system = models.CharField(
        max_length=100, null=True, help_text="The system the character is in"
    )

    shiptype = models.CharField(
        max_length=100,
        null=True,
        db_index=True,
        help_text="The ship the character was flying",
    )

    objects = AFatManager()

    class Meta:  # pylint: disable=too-few-public-methods
        """
        AFat :: Meta
        """

        default_permissions = ()
        unique_together = (("character", "afatlink"),)
        verbose_name = "FAT"
        verbose_name_plural = "FATs"

    def __str__(self) -> str:
        """
        Return the objects string name
        :return:
        :rtype:
        """

        return f"{self.afatlink} - {self.character}"


# ManualAFat Model
class ManualAFat(models.Model):
    """
    ManualAFat
    """

    creator = models.ForeignKey(
        User, related_name="+", on_delete=models.SET(get_sentinel_user)
    )
    afatlink = models.ForeignKey(AFatLink, related_name="+", on_delete=models.CASCADE)
    character = models.ForeignKey(
        EveCharacter, related_name="+", on_delete=models.CASCADE
    )
    created_at = models.DateTimeField(
        blank=True, null=True, help_text="Time this FAT has been added manually"
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        ManualAFat :: Meta
        """

        default_permissions = ()
        verbose_name = "Manual FAT"
        verbose_name_plural = "Manual FATs"

    # Add property for getting the user for a character.
    def __str__(self) -> str:
        """
        Return the objects string name
        :return:
        :rtype:
        """

        return f"{self.afatlink} - {self.character} ({self.creator})"


# AFat Log Model
class AFatLog(models.Model):
    """
    AFatLog
    """

    class Event(models.TextChoices):
        """
        Choices for SRP Status
        """

        CREATE_FATLINK = "CR_FAT_LINK", _("FAT Link Created")
        CHANGE_FATLINK = "CH_FAT_LINK", _("FAT Link Changed")
        DELETE_FATLINK = "RM_FAT_LINK", _("FAT Link Removed")
        REOPEN_FATLINK = "RO_FAT_LINK", _("FAT Link Re-Opened")
        # CREATE_FAT = "CR_FAT", _("FAT Registered")
        DELETE_FAT = "RM_FAT", _("FAT Removed")
        MANUAL_FAT = "CR_FAT_MAN", _("Manual FAT Added")

    log_time = models.DateTimeField(default=timezone.now, db_index=True)
    user = models.ForeignKey(
        User,
        related_name="+",
        null=True,
        blank=True,
        default=None,
        on_delete=models.SET(get_sentinel_user),
    )
    log_event = models.CharField(
        max_length=11,
        blank=False,
        choices=Event.choices,
        default=Event.CREATE_FATLINK,
    )
    log_text = models.TextField()
    fatlink_hash = models.CharField(max_length=254)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        AFatLog :: Meta
        """

        default_permissions = ()
        verbose_name = "AFAT Log"
        verbose_name_plural = "AFAT Logs"



"""
Dashboard related views
"""

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse, JsonResponse
from django.shortcuts import render
from django.urls import reverse

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# Alliance Auth AFAT
from afat import __title__
from afat.helper.views_helper import convert_fatlinks_to_dict, convert_fats_to_dict
from afat.models import AFat, AFatLink

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required()
@permission_required("afat.basic_access")
def overview(request: WSGIRequest) -> HttpResponse:
    """
    Dashboard view
    :param request:
    :type request:
    :return:
    :rtype:
    """

    characters = (
        EveCharacter.objects.select_related("character_ownership")
        .filter(character_ownership__user=request.user, afats__isnull=False)
        .distinct()
    )

    context = {"characters": characters}

    logger.info(f"Module called by {request.user}")

    return render(request, "afat/view/dashboard/dashboard.html", context)


@login_required
@permission_required("afat.basic_access")
def ajax_recent_get_fats_by_character(
    request: WSGIRequest, charid: int
) -> JsonResponse:
    """
    Ajax call :: get all FATs for a given character
    :param request:
    :type request:
    :param charid:
    :type charid:
    :return:
    :rtype:
    """

    character = EveCharacter.objects.get(character_id=charid)

    fats = (
        AFat.objects.select_related_default()
        .filter(character=character)
        .order_by("afatlink__afattime")
        .reverse()[:10]
    )

    character_fat_rows = [
        convert_fats_to_dict(request=request, fat=fat) for fat in fats
    ]

    return JsonResponse(character_fat_rows, safe=False)


@login_required
@permission_required("afat.basic_access")
def ajax_get_recent_fatlinks(request: WSGIRequest) -> JsonResponse:
    """
    Ajax call :: get recent fat links for the dashboard datatable
    :param request:
    :type request:
    :return:
    :rtype:
    """

    fatlinks = (
        AFatLink.objects.select_related_default()
        .annotate_afats_count()
        .order_by("-afattime")[:10]
    )

    fatlink_rows = [
        convert_fatlinks_to_dict(
            request=request,
            fatlink=fatlink,
            close_esi_redirect=reverse("afat:dashboard"),
        )
        for fatlink in fatlinks
    ]

    return JsonResponse(fatlink_rows, safe=False)



# Generated by Django 3.1.5 on 2021-01-11 14:28

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("afat", "0015_afatlink_character"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="aaafat",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the AFAT module"),
                    ("manage_afat", "Can manage the AFAT module"),
                    ("add_fatlink", "Can create FAT Links"),
                    ("stats_corporation_own", "Can see own corporation statistics"),
                    (
                        "stats_corporation_other",
                        "Can see statistics of other corporations",
                    ),
                ),
                "verbose_name": "Alliance Auth AFAT",
            },
        ),
        migrations.AlterModelOptions(
            name="afat",
            options={
                "default_permissions": (),
                "verbose_name": "FAT",
                "verbose_name_plural": "FATs",
            },
        ),
        migrations.AlterModelOptions(
            name="afatdellog",
            options={
                "default_permissions": (),
                "verbose_name": "Delete Log",
                "verbose_name_plural": "Delete Logs",
            },
        ),
        migrations.AlterModelOptions(
            name="afatlink",
            options={
                "default_permissions": (),
                "ordering": ("-afattime",),
                "verbose_name": "FAT Link",
                "verbose_name_plural": "FAT Links",
            },
        ),
        migrations.AlterModelOptions(
            name="afatlinktype",
            options={
                "default_permissions": (),
                "verbose_name": "FAT Link Fleet Type",
                "verbose_name_plural": "FAT Link Fleet Types",
            },
        ),
        migrations.AlterModelOptions(
            name="clickafatduration",
            options={
                "default_permissions": (),
                "verbose_name": "FAT Duration",
                "verbose_name_plural": "FAT Durations",
            },
        ),
        migrations.AlterModelOptions(
            name="manualafat",
            options={
                "default_permissions": (),
                "verbose_name": "Manual FAT Log",
                "verbose_name_plural": "Manual FAT Logs",
            },
        ),
    ]



"""
Template tags
"""

# Django
from django.template.defaulttags import register
from django.utils.translation import gettext as _


@register.filter
def month_name(month_number):
    """
    Template tag :: get month name from month number
    example: {{ event.month|month_name }}
    :param month_number:
    :return:
    """

    month_mapper = {
        1: _("January"),
        2: _("February"),
        3: _("March"),
        4: _("April"),
        5: _("May"),
        6: _("June"),
        7: _("July"),
        8: _("August"),
        9: _("September"),
        10: _("October"),
        11: _("November"),
        12: _("December"),
    }

    return month_mapper[int(month_number)]



# Generated by Django 3.1.4 on 2020-12-24 09:30

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("afat", "0013_basic_module_access_permissions"),
    ]

    operations = [
        migrations.AddField(
            model_name="afatlink",
            name="esi_fleet_id",
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="afatlink",
            name="is_registered_on_esi",
            field=models.BooleanField(
                default=False,
                help_text="Whether this is an ESI fat link is registered on ESI",
            ),
        ),
    ]



# Standard Library
import datetime as dt

# Third Party
from pytz import utc

# Django
from django.test import TestCase
from django.urls import reverse

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth (External Libs)
from app_utils.testing import add_character_to_user, create_user_from_evecharacter

# Alliance Auth AFAT
from afat.models import AFat, AFatLink
from afat.tests.fixtures.load_allianceauth import load_allianceauth
from afat.tests.fixtures.utils import RequestStub
from afat.views.statistics import _calculate_year_stats

MODULE_PATH = "afat.views.statistics"


def response_content_to_str(response) -> str:
    return response.content.decode(response.charset)


class TestStatistics(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()

        # given
        cls.character_1001 = EveCharacter.objects.get(character_id=1001)
        cls.character_1002 = EveCharacter.objects.get(character_id=1002)
        cls.character_1003 = EveCharacter.objects.get(character_id=1003)
        cls.character_1004 = EveCharacter.objects.get(character_id=1004)
        cls.character_1005 = EveCharacter.objects.get(character_id=1005)
        cls.character_1101 = EveCharacter.objects.get(character_id=1101)

        cls.user_without_access, _ = create_user_from_evecharacter(
            cls.character_1001.character_id
        )

        cls.user_with_basic_access, _ = create_user_from_evecharacter(
            cls.character_1002.character_id, permissions=["afat.basic_access"]
        )

        add_character_to_user(cls.user_with_basic_access, cls.character_1101)

        cls.user_with_manage_afat, _ = create_user_from_evecharacter(
            cls.character_1003.character_id,
            permissions=["afat.basic_access", "afat.manage_afat"],
        )

        cls.user_with_stats_corporation_other, _ = create_user_from_evecharacter(
            cls.character_1004.character_id,
            permissions=["afat.basic_access", "afat.stats_corporation_other"],
        )

        cls.user_with_stats_corporation_own, _ = create_user_from_evecharacter(
            cls.character_1005.character_id,
            permissions=["afat.basic_access", "afat.stats_corporation_own"],
        )

        # Generate some FAT links and FATs
        afat_link_april_1 = AFatLink.objects.create(
            fleet="April Fleet 1",
            hash="1231",
            creator=cls.user_with_basic_access,
            character=cls.character_1001,
            afattime=dt.datetime(2020, 4, 1, tzinfo=utc),
        )
        afat_link_april_2 = AFatLink.objects.create(
            fleet="April Fleet 2",
            hash="1232",
            creator=cls.user_with_basic_access,
            character=cls.character_1001,
            afattime=dt.datetime(2020, 4, 15, tzinfo=utc),
        )
        afat_link_september = AFatLink.objects.create(
            fleet="September Fleet",
            hash="1233",
            creator=cls.user_with_basic_access,
            character=cls.character_1001,
            afattime=dt.datetime(2020, 9, 1, tzinfo=utc),
        )

        AFat.objects.create(
            character=cls.character_1101, afatlink=afat_link_april_1, shiptype="Omen"
        )
        AFat.objects.create(
            character=cls.character_1001, afatlink=afat_link_april_1, shiptype="Omen"
        )
        AFat.objects.create(
            character=cls.character_1002, afatlink=afat_link_april_1, shiptype="Omen"
        )
        AFat.objects.create(
            character=cls.character_1003, afatlink=afat_link_april_1, shiptype="Omen"
        )
        AFat.objects.create(
            character=cls.character_1004, afatlink=afat_link_april_1, shiptype="Omen"
        )
        AFat.objects.create(
            character=cls.character_1005, afatlink=afat_link_april_1, shiptype="Omen"
        )

        AFat.objects.create(
            character=cls.character_1101, afatlink=afat_link_april_2, shiptype="Omen"
        )
        AFat.objects.create(
            character=cls.character_1004, afatlink=afat_link_april_2, shiptype="Thorax"
        )
        AFat.objects.create(
            character=cls.character_1002, afatlink=afat_link_april_2, shiptype="Thorax"
        )
        AFat.objects.create(
            character=cls.character_1003, afatlink=afat_link_april_2, shiptype="Omen"
        )

        AFat.objects.create(
            character=cls.character_1001, afatlink=afat_link_september, shiptype="Omen"
        )
        AFat.objects.create(
            character=cls.character_1004,
            afatlink=afat_link_september,
            shiptype="Guardian",
        )
        AFat.objects.create(
            character=cls.character_1005, afatlink=afat_link_september, shiptype="Omen"
        )

    def test_should_only_show_my_chars_and_only_those_with_fat_links(self):
        # when
        result = _calculate_year_stats(RequestStub(self.user_with_basic_access), 2020)

        # then
        self.assertListEqual(
            result, [("Clark Kent", {4: 2}, 1002), ("Lex Luther", {4: 2}, 1101)]
        )

    def test_should_show_statistics_dashboard(self):
        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        url = reverse("afat:statistics_overview")
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_statistics_dashboard_for_year(self):
        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        url = reverse("afat:statistics_overview", kwargs={"year": 2020})
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_statistics_dashboard_for_user_with_stats_corporation_other(
        self,
    ):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse("afat:statistics_overview")
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_statistics_dashboard_for_user_with_stats_corporation_own(self):
        # given
        self.client.force_login(self.user_with_stats_corporation_own)

        # when
        url = reverse("afat:statistics_overview")
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_not_show_statistics_dashboard_for_user_without_access(self):
        # given
        self.client.force_login(self.user_without_access)

        # when
        url = reverse("afat:statistics_overview")
        res = self.client.get(url)

        # then
        self.assertNotEqual(res.status_code, 200)
        self.assertEqual(res.status_code, 302)
        self.assertEqual(
            res.url, "/account/login/?next=/fleet-activity-tracking/statistics/"
        )

    def test_should_show_own_character_stats(self):
        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        url = reverse(
            "afat:statistics_character",
            kwargs={
                "charid": self.user_with_basic_access.profile.main_character.character_id,
                "year": 2020,
                "month": 4,
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_other_character_stats_for_user_with_stats_corporation_own(
        self,
    ):
        # given
        self.client.force_login(self.user_with_stats_corporation_own)

        # when
        url = reverse(
            "afat:statistics_character",
            kwargs={
                "charid": self.user_with_basic_access.profile.main_character.character_id,
                "year": 2020,
                "month": 4,
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_other_character_stats_for_user_with_stats_corporation_other(
        self,
    ):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse(
            "afat:statistics_character",
            kwargs={
                "charid": self.user_with_basic_access.profile.main_character.character_id,
                "year": 2020,
                "month": 4,
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_other_character_stats_for_user_with_manage_afat(self):
        # given
        self.client.force_login(self.user_with_manage_afat)

        # when
        url = reverse(
            "afat:statistics_character",
            kwargs={
                "charid": self.user_with_basic_access.profile.main_character.character_id,
                "year": 2020,
                "month": 4,
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_not_show_other_character_stats_for_user(self):
        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        url = reverse(
            "afat:statistics_character",
            kwargs={
                "charid": self.user_with_stats_corporation_other.profile.main_character.character_id,
                "year": 2020,
                "month": 4,
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 302)

    def test_should_show_own_corp_stats_for_user_with_stats_corporation_own(self):
        # given
        self.client.force_login(self.user_with_stats_corporation_own)

        # when
        url = reverse(
            "afat:statistics_corporation",
            kwargs={
                "corpid": self.user_with_stats_corporation_own.profile.main_character.corporation_id
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_other_corp_stats_for_user_with_stats_corporation_other(self):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse(
            "afat:statistics_corporation",
            kwargs={
                "corpid": self.user_with_basic_access.profile.main_character.corporation_id
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_other_corp_stats_for_user_with_manage_afat(self):
        # given
        self.client.force_login(self.user_with_manage_afat)

        # when
        url = reverse(
            "afat:statistics_corporation",
            kwargs={
                "corpid": self.user_with_basic_access.profile.main_character.corporation_id
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_not_show_own_corp_stats_for_user(self):
        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        url = reverse(
            "afat:statistics_corporation",
            kwargs={
                "corpid": self.user_with_basic_access.profile.main_character.corporation_id
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 302)

    def test_should_not_show_other_corp_stats_for_user(self):
        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        url = reverse(
            "afat:statistics_corporation",
            kwargs={
                "corpid": self.user_with_stats_corporation_other.profile.main_character.corporation_id
            },
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 302)

    def test_should_show_all_corp_stats_for_user_with_stats_corporation_other(self):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse(
            "afat:statistics_corporation",
            kwargs={"corpid": 2002, "year": 2020},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_all_corp_stats_with_month_for_user_with_stats_corporation_other(
        self,
    ):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse(
            "afat:statistics_corporation",
            kwargs={"corpid": 2002, "year": 2020, "month": 4},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_not_show_all_corp_stats_for_user_with_stats_corporation_own(self):
        # given
        self.client.force_login(self.user_with_stats_corporation_own)

        # when
        url = reverse("afat:statistics_corporation", kwargs={"corpid": 2002})
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 302)
        self.assertEqual(res.url, "/fleet-activity-tracking/")

    def test_should_show_all_alliance_stats_with_for_user_with_stats_corporation_other(
        self,
    ):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse(
            "afat:statistics_alliance",
            kwargs={"allianceid": 3001},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_all_alliance_stats_with_for_user_with_manage_afat(
        self,
    ):
        # given
        self.client.force_login(self.user_with_manage_afat)

        # when
        url = reverse(
            "afat:statistics_alliance",
            kwargs={"allianceid": 3001},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_all_alliance_stats_with_year_for_user_with_stats_corporation_other(
        self,
    ):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse(
            "afat:statistics_alliance",
            kwargs={"allianceid": 3001, "year": 2020},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_all_alliance_stats_with_year_for_user_with_manage_afat(
        self,
    ):
        # given
        self.client.force_login(self.user_with_manage_afat)

        # when
        url = reverse(
            "afat:statistics_alliance",
            kwargs={"allianceid": 3001, "year": 2020},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_all_alliance_stats_with_month_for_user_with_stats_corporation_other(
        self,
    ):
        # given
        self.client.force_login(self.user_with_stats_corporation_other)

        # when
        url = reverse(
            "afat:statistics_alliance",
            kwargs={"allianceid": 3001, "year": 2020, "month": 4},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_show_all_alliance_stats_with_month_for_user_with_manage_afat(
        self,
    ):
        # given
        self.client.force_login(self.user_with_manage_afat)

        # when
        url = reverse(
            "afat:statistics_alliance",
            kwargs={"allianceid": 3001, "year": 2020, "month": 4},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 200)

    def test_should_not_show_all_alliance_stats_for_user(
        self,
    ):
        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        url = reverse(
            "afat:statistics_alliance",
            kwargs={"allianceid": 3001},
        )
        res = self.client.get(url)

        # then
        self.assertEqual(res.status_code, 302)



# Generated by Django 2.2.14 on 2020-08-16 20:42

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("afat", "0004_clickfatduration"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="afatlink",
            options={
                "ordering": ("-afattime",),
                "permissions": (
                    (
                        "manage_afat",
                        "Can manage the Another Fleet Activity Tracking module",
                    ),
                    ("stats_corp_own", "Can see own corp stats"),
                    ("stats_corp_other", "Can see stats of other corps."),
                    (
                        "stats_char_other",
                        "Can see stats of characters not associated with current user.",
                    ),
                ),
            },
        ),
        migrations.AddField(
            model_name="afat",
            name="deleted_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="afatlink",
            name="deleted_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
